/*
 * SplitSet.hpp
 *
 *  Created on: Sep 18, 2013
 *      Author: ckeme_01
 */

#ifndef SPLITSET_HPP_
#define SPLITSET_HPP_

// C++ header
#include <vector>
#include <memory>
#include <string>

// MDAT header


namespace MDAT
{

template<typename DataType>
class SplitSet
{
private:
	std::vector<std::shared_ptr<DataType> > _set;
	int _id;
	std::vector<int> _pattern;

public:

	typedef DataType value_type;

	SplitSet();
	SplitSet(const std::vector<int> &pattern);


	~SplitSet();


	/**@{*/
	//! \name Operators

	/**
	 * \brief Operator to access the sequence
	 * @param index The sequence position to return.
	 * @return Pointer to the sequence.
	 */
	DataType &operator[](unsigned int index)
	{
		return *_set[index];
	}

	/**
	 * \overload
	 */
	const DataType &operator[](unsigned int index) const
	{
		return *_set[index];
	}
	/**@}*/



	void push_back(DataType* set)
	{
		_set.push_back(std::shared_ptr<DataType>(set));
	}

	size_t
	size() const
	{
		return _set.size();
	}

	int
	id() const
	{
		return _id;
	}

	void
	id(int val)
	{
		_id=val;
	}

	std::vector<int>
	pattern() const
	{
		return _pattern;
	}

	void
	pattern(const std::vector<int> &new_pattern)
	{
		_pattern=new_pattern;
	}


	template<typename SplitSetType>
	friend std::ostream& operator<< (std::ostream &out, const SplitSet<SplitSetType> &seqSet);



};

template<typename SplitSetType>
std::ostream& operator<< (std::ostream &out, const SplitSet<SplitSetType> &seqSet)
{
	size_t n_sets=seqSet.size();
	size_t n_seqs = seqSet[0].size();
	for (size_t j=0; j<n_seqs; ++j)
	{
		for (size_t i=0; i<n_sets; ++i)
		{
			out << " " << seqSet[i][j].sequence();
		}
		out << std::endl;
	}
	return out;
}


template<typename DataType>
SplitSet<DataType>::SplitSet():_set(), _id(-1), _pattern()
{

}


template<typename DataType>
SplitSet<DataType>::SplitSet(const std::vector<int> &pattern):_set(), _id(-1), _pattern(pattern)
{
	size_t n_pieces=_pattern.size()*2+1;
	_set.reserve(n_pieces);
	for (size_t i=0; i<n_pieces; ++i)
		_set.push_back(std::shared_ptr<DataType>(new DataType()));
}

template<typename DataType>
SplitSet<DataType>::~SplitSet()
{

}



}
#endif /* SPLITSET_HPP_ */
