/*
 * other_seq.hpp
 *
 *  Created on: Jun 9, 2013
 *      Author: Carsten Kemena
 *
 * This file is part of MDAT.
 *
 * MDAT is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MDAT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with MDAT.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef OTHER_SEQ_HPP_
#define OTHER_SEQ_HPP_

#include "SequenceSet.hpp"


template<typename SequenceSetType>
double
avg_size(const SequenceSetType &seqSet)
{
	size_t n_seqs = seqSet.n_seqs();
	double length = 0;
	for (size_t i=0; i<n_seqs; ++i)
		length+= seqSet->size();
	return length/n_seqs;
}


template<typename SequenceSetType>
size_t
max_size(const SequenceSetType &seqSet)
{
	size_t max_len = 0;
	size_t n_seqs = seqSet.n_seqs();
	for (size_t i=0; i<n_seqs; ++i)
		if (seqSet[i].size() > max_len)
			max_len=seqSet[i].size();
	return max_len;
}


template<typename SequenceType, typename MemoryType>
bool
check_set(const SequenceSet<SequenceType, MemoryType> &set)
{
	size_t n_seqs = set.n_seqs();
	size_t j, len;
	size_t *val_counting = new size_t[256];
	for (j=0; j<256; ++j)
		val_counting[j] = 0;

	// counting occurrences of characters
	for (size_t i=0; i<n_seqs; ++i)
	{
		const SequenceType &seq = set[i];
		len=seq.size();
		for (j=0; j<len; ++j)
			++val_counting[static_cast<int>(seq[j])];
	}
	// see if strange character has been found
	for (j=0; j<45; ++j)
	{
		if (val_counting[j] != 0)
			return false;
	}
	for (j=46; j<65; ++j)
	{
		if (val_counting[j] != 0)
			return false;
	}
	for (j=91; j<97; ++j)
	{
		if (val_counting[j] != 0)
			return false;
	}
	for (j=123; j<256; ++j)
	{
		if (val_counting[j] != 0)
			return false;
	}
	return true;
}



#endif /* OTHER_SEQ_HPP_ */
