/*
 * consistency_aln.cpp
 *
 *  Created on: Sep 20, 2013
 *      Author: Carsten Kemena
 */

#include "consistency_aln.hpp"

namespace MDAT
{


void
nw_dyn_consistency(size_t dim1, size_t dim2, Matrix<std::pair<int, char> > &matrix)
{
	++dim1;
	++dim2;
	int score;
	size_t i,j;
	for (i=0; i<dim1; ++i)
		matrix[i][0].first=0;
	for (j=1; j<dim2; ++j)
		matrix[0][j].first=0;

	for (i=1; i<dim1; ++i)
	{
		for (j=1; j<dim2; ++j)
		{
			score =	matrix[i][j].first;

			if (matrix[i-1][j].first > matrix[i][j-1].first)
			{
				matrix[i][j].first = matrix[i-1][j].first;
				matrix[i][j].second = 'i';
			}
			else
			{
				matrix[i][j].first = matrix[i][j-1].first;
				matrix[i][j].second = 'j';
			}
			if ((score != 0) && (matrix[i-1][j-1].first+score >= matrix[i][j].first))
			{
				matrix[i][j].first = matrix[i-1][j-1].first+score;
				matrix[i][j].second = 'm';
			}
		}
	}
}


void
nw_dyn_consistency_traceback(size_t i, size_t j, const Matrix<std::pair<int, char> > &matrix, std::string &edit_string1, std::string &edit_string2)
{
	edit_string1.clear();
	edit_string1.reserve(std::max(i,j));
	edit_string2.clear();
	edit_string2.reserve(std::max(i,j));

	while ((i!=0) && (j!=0))
	{
		if (matrix[i][j].second == 'm')
		{
			--i;
			--j;
			edit_string1.push_back('m');
			edit_string2.push_back('m');
		}
		else if (matrix[i][j].second == 'i')
		{
			--i;
			edit_string1.push_back('m');
			edit_string2.push_back('-');
		}
		else
		{
			--j;
			edit_string1.push_back('-');
			edit_string2.push_back('m');
		}
	}

	if (i>0)
	{
		edit_string1.append(i,'m');
		edit_string2.append(i,'-');
	}
	else if (j>0)
	{
		edit_string1.append(j,'-');
		edit_string2.append(j,'m');
	}
}


void
gotoh_dyn_consistency(int dim1, int dim2, MatrixStack<3, std::pair<int, char> > &matrices, int gop, int gep)
{
	// initialization
	Matrix<std::pair<int, char> > &mat_m = matrices[0];
	Matrix<std::pair<int, char> > &mat_h = matrices[1];
	Matrix<std::pair<int, char> > &mat_v = matrices[2];

	size_t l_seq1 = dim1;
	size_t l_seq2 = dim2;
	int score;
	// 0=match, 1=insert, 2=deletion


	mat_m[0][0].first= 0;
	mat_h[0][0].first= 0;
	mat_v[0][0].first= 0;
	size_t i,j;
	for (i=1; i<l_seq1; ++i)
	{
		mat_v[i][0].first=INT_MIN+1000;
		mat_m[i][0].first=mat_h[i][0].first=mat_h[i-1][0].first+gep;
	}
	for (j=1; j<l_seq2; ++j)
	{
		mat_m[0][j].first=mat_v[0][j].first=mat_v[0][j-1].first+gep;
		mat_h[0][j].first=INT_MIN+1000;
	}


	for (i=1; i<l_seq1; ++i)
	{
		for (j=1; j<l_seq2; ++j)
		{
			score = mat_m[i][j].first;
			//calculate insert value

			if (mat_v[i-1][j].first > mat_m[i-1][j].first+gop)
			{
				mat_v[i][j].second = 'i';
				mat_v[i][j].first = mat_v[i-1][j].first+gep;
			}else
			{
				mat_v[i][j].second = 'm';
				mat_v[i][j].first = mat_m[i-1][j].first+gop+gep;
			}

			//calculate deletion value
			if (mat_h[i][j-1].first > mat_m[i][j-1].first+gop)
			{
				mat_h[i][j].second = 'd';
				mat_h[i][j].first = mat_h[i][j-1].first+gep;
			} else
			{
				mat_h[i][j].second = 'm';
				mat_h[i][j].first = mat_m[i][j-1].first+gop+gep;
			}


			//calculate match value
			if (mat_v[i][j] > mat_h[i][j])
			{
				mat_m[i][j].second = 'i';
				mat_m[i][j].first = mat_v[i][j].first;
			}
			else
			{
				mat_m[i][j].second = 'd';
				mat_m[i][j].first = mat_h[i][j].first;
			}

			if ((score!=0) && (mat_m[i-1][j-1].first + score >= mat_m[i][j].first))
			{
				mat_m[i][j].second = 'm';
				mat_m[i][j].first = mat_m[i-1][j-1].first + score;
			}
		}
	}
}

void
gotoh_dyn_consistency_traceback(size_t i, size_t j, MatrixStack<3,std::pair<int, char> > &matrices, std::string &edit_string1, std::string &edit_string2)
{
	edit_string1.clear();
	edit_string1.reserve(std::max(i,j));
	edit_string2.clear();
	edit_string2.reserve(std::max(i,j));

	char state='c';
	int mat = 0;
	while ((i!=0) && (j!=0))
	{
		state = matrices[mat][i][j].second;
		if (mat==0)
		{
			if (state=='m')
			{
				--i;
				--j;
				edit_string1.push_back('m');
				edit_string2.push_back('m');
			}
			else
			{
				if (state=='i')
					mat=2;
				else
					mat=1;
			}
		}
		else
		{
			if (mat==2)
			{
				--i;
				edit_string1.push_back('m');
				edit_string2.push_back('-');
			}
			else
			{
				--j;
				edit_string1.push_back('-');
				edit_string2.push_back('m');
			}
			if (state=='m')
				mat = 0;
		}
	}

	if (i>0)
	{
		edit_string1.append(i,'m');
		edit_string2.append(i,'-');
	}
	if (j>0)
	{
		edit_string1.append(j,'-');
		edit_string2.append(j,'m');
	}

}




}




