/*
// * My_IO_Exception.h
 *
 *  Created on: Feb 23, 2012
 *      Author: Carsten Kemena
 *
 * This file is part of MDAT.
 *
 * MDAT is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MDAT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with MDAT.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <stdexcept>

#ifndef MY_IO_EXCEPTION_H_
#define MY_IO_EXCEPTION_H_


/**
 * \brief Exception thrown for failed opening of a file.
 */
class MDAT_IO_Exception : public std::runtime_error
{
public:
	/**
	 * \brief Message to return with what()
	 * @param message The message.
	 */
	MDAT_IO_Exception(const char *message) : std::runtime_error(message)
	{}

	MDAT_IO_Exception(const MDAT_IO_Exception&) = default;
	MDAT_IO_Exception(MDAT_IO_Exception&&) = default;
	virtual ~MDAT_IO_Exception() noexcept
			{}
};


/**
 * \brief Exception thrown when trying to read a file (e.g. format problems)
 */
class Alignment_Exception : public std::runtime_error
{
public:
	/**
	 * \brief Message to return with what()
	 * @param message The message.
	 */
	Alignment_Exception(const char *message) : std::runtime_error(message)
	{}

	Alignment_Exception(Alignment_Exception&&) =default;
	Alignment_Exception(const Alignment_Exception&) =default;
	virtual ~Alignment_Exception() noexcept
			{}
};


#endif /* MY_IO_EXCEPTION_H_ */
