/*
 * Matrix1Line.hpp
 *
 *  Created on: 3 Oct 2013
 *      Author: ck
 */

#ifndef MATRIX1LINE_HPP_
#define MATRIX1LINE_HPP_



// C header
#include <cstdlib>


// C++ header
#include <vector>

namespace MDAT {

/**
 * \brief A simple class to produce 2 dimensional matrices.
 */
template <typename DataType>
class Matrix1Line
{


private:
	size_t _dim1, _dim2;
	std::vector<DataType> _matrix;

public:
	typedef DataType value_type;


	Matrix1Line():_dim1(0), _dim2(0)
	{}

	/**
	 * \brief Constructor setting the size.
	 * @param dim1 Size of the first dimension.
	 * @param dim2 Size of the second dimension.
	 */
	Matrix1Line(size_t dim1, size_t dim2);

	/**
	 * \brief Constructor allowing initialization of fields.
	 * @param dim1 Size of the first dimension.
	 * @param dim2 Size of the second dimension.
	 * @param init Value to initalize the fields with.
	 */
	Matrix1Line(size_t dim1, size_t dim2, DataType init);

	/**
	 * \brief Destructor
	 */
	virtual ~Matrix1Line()
	{

	}

	/**
	 * \brief Access operator
	 * @param index The index to acess.
	 * @return Reference to the field.
	 */
	DataType * operator[](unsigned int index)
	{
		return &(_matrix.data()[index*_dim2]);
	}

	/**
	 * \brief Access operator
	 * @param index The index to acess.
	 * @return Reference to the field.
	 */
	DataType *operator[](unsigned int index) const
	{
		return &(_matrix.data()[index*_dim2]);
	}

	typename std::vector<DataType>::iterator begin(size_t id)
	{
		return _matrix.begin()+id*_dim2;
	}

	typename std::vector<DataType>::iterator end(size_t id)
	{
		if (id == (_dim1-1))
			return _matrix.end();
		else
			return _matrix.begin()+((id+1)*_dim2);
	}

	/**
	 * \brief Resizes the matrix to the new dimensions.
	 * @param dim_1 The first dimension
	 * @param dim_2 The second diemension
	 */
	void resize(size_t dim_1, size_t dim_2)
	{
		_dim1=dim_1;
		_dim2=dim_2;
		size_t dim= dim_1*dim_2;
		for (size_t i=0; i<dim; ++i)
			_matrix[i].resize(dim);
	}

	/**
	 * \brief Returns the size of the first dimension.
	 * @return The size of the first dimension.
	 */
	size_t dim1() const
	{
		return _dim1;
	}

	size_t size() const
	{
		return _matrix.size();
	}
	/**
	 * \brief Returns the size of the second dimension.
	 * @return The size of the second dimension.
	 */
	size_t dim2() const
	{
		return (_dim1 > 0) ? _dim2 : 0;
	}

	void
	fill(const DataType &value)
	{
		size_t dim = _dim1*_dim2;
		for (size_t i=0; i<dim; ++i)
			_matrix[i] = value;
	}
};


template <typename DataType>
Matrix1Line<DataType>::Matrix1Line(size_t dim_1, size_t dim_2, DataType init):_dim1(dim_1), _dim2(dim_2)
{
	_matrix.resize(dim_1*dim_2, init);
}


template <typename DataType>
Matrix1Line<DataType>::Matrix1Line(size_t dim_1, size_t dim_2):_dim1(dim_1), _dim2(dim_2)
{
	_matrix.resize(dim_1*dim_2);
}

}


#endif /* MATRIX1LINE_HPP_ */
