/*
 * filesystem.h
 *
 *  Created on: Oct 9, 2011
 *      Author: Carsten Kemena
 *
 * This file is part of MDAT.
 *
 * MDAT is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MDAT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with MDAT.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef FILESYSTEM_H_
#define FILESYSTEM_H_


// C header
#include <cstdio>
#include <cstring>
#include <cerrno>

// C++ header
#include <string>
#include <vector>
#include <exception>

// MDAT header
#include "./MDAT_Exceptions.hpp"

/*! \file filesystem.h
    \brief Contains some filesystem functions.
*/


namespace MDAT {

/**
 * Open a file. Will throw an exception when an error occurs.
 * @param name_f Name of the file.
 * @param mode Mode to open it with
 * @return The File pointer.
 */
inline FILE *
my_fopen(std::string name_f, std::string mode)
{
	FILE *name_F = fopen(name_f.c_str(), mode.c_str());
	if (name_F == nullptr)
	{
		throw MDAT_IO_Exception(strerror(errno));
	}
	else
		return name_F;
}

} /* namespace MDAT */

//void
//get_files_in_dir(const std::string &directory, std::vector<std::string> &files, std::vector<std::string> ending, bool recursive);

#endif /* FILESYSTEM_H_ */
