/*
 * mda_pdf_out.hpp
 *
 *  Created on: 22 Sep 2013
 *      Author: ck
 */

#ifndef MDA_PDF_OUT_HPP_
#define MDA_PDF_OUT_HPP_

// C++ header
#include <string>
#include <vector>
#include <cmath>
#include <random>

// Cairo headers
#include <cairo/cairo.h>
#include <cairo/cairo-pdf.h>

// Boost header
#include <boost/lexical_cast.hpp> // needed because to_string is not supported in Cygwin

// MDAT

#include "../../lib/Sequence/ProteinSequenceSet.hpp"
#include "../../lib/utils/Matrix.hpp"


/**
 * graphical output options.
 */
struct Opt_graphic
{
public:
	Opt_graphic():graphic_f(""), show_sim_mat(false), show_sequence_names(false), color_by_similarity(false)
	{}

	Opt_graphic(const Opt_graphic&)=delete;
	Opt_graphic(Opt_graphic&&)=delete;

	std::string graphic_f;
	bool show_sim_mat;
	bool show_sequence_names;
	bool color_by_similarity;
};


/*
void
draw_domain(cairo_t *ctx, const MDAT::Domain &dom);
*/

/**
 * \brief Writes the matrix to a PDF.
 * @param seqSet The sequence set.
 * @param dom_mat The domain similarity matrix.
 * @param out_f The output file
 * @param show_sim_mat Parameter to plot the similarity matrix.
 * @param show_seq_names Parameter to show the sequence names in a list.
 */
void
write_mda_as_pdf(const MDAT::ProteinSequenceSet<MDAT::Default> &seqSet, const MDAT::Matrix<short> &dom_mat, const Opt_graphic &graphic_opts);



#endif /* MDA_PDF_OUT_HPP_ */
