/*
 * options.hpp
 *
 *  Created on: Jul 26, 2013
 *      Author: ckeme_01
 */

#ifndef OPTIONS_HPP_
#define OPTIONS_HPP_

// C++ header
#include<string>
#include<vector>

//MDAT header
#include "../../lib/utils/Matrix.hpp"

/**
 * General program options.
 */
struct Opt_general
{
	Opt_general() : sequence_f(""), domain_f(""), mda_out_f(""), msa_out_f(""), out_msa_format(""), domain_program(""), domain_database(""), n_threads(0)
	{}
	std::string sequence_f;
	std::string domain_f;
	std::string mda_out_f;
	std::string msa_out_f;
	std::string out_msa_format;
	std::string domain_program;
	std::string domain_database;
	int n_threads;
};

/**
 * \MSA construction options
 */
struct Opt_msa
{
	Opt_msa() : gop(0), gep(0), simmat_name(""), sim_mat(nullptr)
	{}
	Opt_msa& operator=(const Opt_msa&) = delete;
	Opt_msa& operator=(Opt_msa&&) = delete;

	Opt_msa(const Opt_msa&) = delete;
	Opt_msa(Opt_msa&&) = delete;

	int gop;
	int gep;
	std::string simmat_name;
	MDAT::Matrix<int> *sim_mat;
	bool relax;
};


/**
 * domain options
 */
struct Opt_dom
{
	Opt_dom() : use_envelope(false), no_merging(false), refine_boundaries(false), no_rep_col(false)
	{}

	bool use_envelope;
	bool no_merging;
	bool refine_boundaries;
	bool no_rep_col;

};


#endif /* OPTIONS_HPP_ */
