/*
 * Align_test.hpp
 *
 *  Created on: Oct 10, 2013
 *      Author: ckeme_01
 */

#ifndef ALIGN_TEST_HPP_
#define ALIGN_TEST_HPP_


#include "../lib/Sequence/SequenceSet.hpp"
#include "../lib/align/seq_align.hpp"
#include "../lib/utils/MatrixStack.hpp"



// C header
#include <cstdlib>

// CxxTest header
#include <cxxtest/TestSuite.h>


class Align_Test : public CxxTest::TestSuite
{
private:
	MDAT::Matrix<int>*
	read_similarity_matrix(const char *mat_f)
	{
		FILE *mat_F = fopen(mat_f, "r");
		const int LINE_LENGTH=1001;
		char line[LINE_LENGTH];
		while (fgets(line, LINE_LENGTH, mat_F) != NULL)
		{
			if (line[0]!='#')
				break;
		}


		MDAT::Matrix<int> *sim_mat_p =new MDAT::Matrix<int>(26,26,-999);
		MDAT::Matrix<int> &sim_mat = *sim_mat_p;
		int key[256];
		char *tmp=strtok(line, " ");
		int pos =0;
		key[pos]=toupper(tmp[0])-65;
		while ((tmp=strtok(NULL, " ")) != NULL)
		{
			key[++pos]=toupper(tmp[0])-65;
			if ((key[pos]<0) || (key[pos]>25))
				key[pos]=-1;
		}

		int row_id;
		int col_id;
		while (fgets(line, LINE_LENGTH, mat_F) != NULL)
		{
			col_id=-1;
			tmp=strtok(line, " \n");
			if (tmp[0]=='*')
				continue;
			row_id=toupper(tmp[0])-65;
			while ((tmp=strtok(NULL, " \n")) != NULL)
			{
				if (key[++col_id] != -1)
					sim_mat[row_id][key[col_id]]=sim_mat[key[col_id]][row_id]=atoi(tmp);
			}
		}
		fclose(mat_F);
		return sim_mat_p;
	}


public:

	void test_gotoh_match()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PTPAPGLFPSPIPASYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PTPAPGLFPSPIPASYLFP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		gotoh_align(set[0].size(), set[1].size(), matrices, -11, -1);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "mmmmmmmmmmmmmmmmmmm");
		TS_ASSERT_EQUALS(edit_string2, "mmmmmmmmmmmmmmmmmmm");
	}

	void test_gotoh_gap_seq1()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PTPAPGLFPSPIPAS"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PTPAPGLFPSPIPASYLFP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		gotoh_align(set[0].size(), set[1].size(), matrices, -11, -1);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "----mmmmmmmmmmmmmmm");
		TS_ASSERT_EQUALS(edit_string2, "mmmmmmmmmmmmmmmmmmm");

	}

	void test_gotoh_gap_seq2()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PTPAPGLFPSPIPASYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PTPAPGLFPSPIPAS"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		gotoh_align(set[0].size(), set[1].size(), matrices, -11, -1);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "mmmmmmmmmmmmmmmmmmm");
		TS_ASSERT_EQUALS(edit_string2, "----mmmmmmmmmmmmmmm");
	}

	void test_gotoh_gap_seq2_start()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PTPAPGLFPSPIPASYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PGLFPSPIPASYLFP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		gotoh_align(set[0].size(), set[1].size(), matrices, -11, -1);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "mmmmmmmmmmmmmmmmmmm");
		TS_ASSERT_EQUALS(edit_string2, "mmmmmmmmmmmmmmm----");
	}
	void test_gotoh_gap_seq1_start()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PGLFPSPIPASYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PTPAPGLFPSPIPASYLFP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		gotoh_align(set[0].size(), set[1].size(), matrices, -11, -1);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "mmmmmmmmmmmmmmm----");
		TS_ASSERT_EQUALS(edit_string2, "mmmmmmmmmmmmmmmmmmm");
	}

	void test_gotoh_gap_middle()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PGLFPSAIPASYLYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PGLFPIPASYLEEYLFP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		gotoh_align(set[0].size(), set[1].size(), matrices, -11, -1);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "mmmm--mmmmmmmmmmmmm");
		TS_ASSERT_EQUALS(edit_string2, "mmmmmmmmmmmm--mmmmm");
		set[0].insert_gaps(edit_string1);
		set[1].insert_gaps(edit_string2);
	}


	void test_whatever()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "MRRNPSTFKYIPLRIDFMSEVPLPE"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "MSDLQDQEPSIIINGNLEPVGEPDIVEETEVVAQETQETQDADKPKKKVAFTGLEEDGETEEEKRKREFEEGGGLPEQPLNPDFSKLNPLSAEIINRQ"));
		set.add_seq(new MDAT::ProteinSequence("seq3", "MNDALEIMKRQ"));
		set.add_seq(new MDAT::ProteinSequence("seq4", "MADEHRQP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		gotoh_align(set[0].size(), set[1].size(), matrices, -11, -1);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		set[0].insert_gaps(edit_string1);
		set[1].insert_gaps(edit_string2);
	}


	void test_gotoh_alignExceptRegion()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PTPAPGLFPSPIPASYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PTPAPGLFPSPIPASYLFP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		std::vector<int> regions1(20,0);
		std::vector<int> regions2(20,0);
		regions1[4] =1;
		regions1[5] =2;
		regions1[6] =1;
		regions2[10] =1;
		regions2[11] =2;
		regions2[12] =1;
		gotoh_alignExceptRegion(set[0].size(), set[1].size(), matrices, -11, -1, regions1, regions2);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "mmmmmmmmmmmmmmmmmmm");
		TS_ASSERT_EQUALS(edit_string2, "mmmmmmmmmmmmmmmmmmm");
	}

	void test_gotoh_alignExceptRegion2()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PTPAPGLFPSPIPASYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PTPAPGLFPSPIPASYLFP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		std::vector<int> regions1(20,0);
		std::vector<int> regions2(20,0);
		regions1[4] =1;
		regions1[5] =2;
		regions1[6] =1;
		regions2[4] =1;
		regions2[5] =2;
		regions2[6] =1;
		gotoh_alignExceptRegion(set[0].size(), set[1].size(), matrices, -11, -1, regions1, regions2);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "mmmmmmmmmmmmm---mmmmmm");
		TS_ASSERT_EQUALS(edit_string2, "mmmmmmmmmmmmmmmm---mmm");
	}

	void test_gotoh_alignExceptRegion3()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PTPAPGLFPSPIPASYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PTPAPGLFPSPIPASYLFP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		std::vector<int> regions1(20,0);
		std::vector<int> regions2(20,0);
		regions1[17] =1;
		regions1[18] =2;
		regions1[19] =1;
		regions2[17] =1;
		regions2[18] =2;
		regions2[19] =1;
		gotoh_alignExceptRegion(set[0].size(), set[1].size(), matrices, -11, -1, regions1, regions2);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "---mmmmmmmmmmmmmmmmmmm");
		TS_ASSERT_EQUALS(edit_string2, "mmm---mmmmmmmmmmmmmmmm");
	}

	void test_gotoh_alignExceptRegion4()
	{
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PTPAPGLFPSPIPASYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PTPAPGLFPSPIPASYLFP"));
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat);
		std::vector<int> regions1(20,0);
		std::vector<int> regions2(20,0);
		regions1[1] =1;
		regions1[2] =2;
		regions1[3] =1;
		regions2[1] =1;
		regions2[2] =2;
		regions2[3] =1;
		gotoh_alignExceptRegion(set[0].size(), set[1].size(), matrices, -11, -1, regions1, regions2);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		TS_ASSERT_EQUALS(edit_string1, "mmmmmmmmmmmmmmmmmmm---");
		TS_ASSERT_EQUALS(edit_string2, "mmmmmmmmmmmmmmmm---mmm");
	}


	void test_createCalMat()
	{
		std::vector<MDAT::Region> reg1, reg2;
		reg1.push_back(MDAT::Region(1,3,8));
		reg2.push_back(MDAT::Region(0,3,8));
		/*reg2.push_back(MDAT::Region(2,11,13));
		reg1.push_back(MDAT::Region(3,10,15));
		reg1.push_back(MDAT::Region(4,17,18));*/
		MDAT::Matrix<int> mat;
		MDAT::ProteinSequenceSet<MDAT::Default> set;
		set.add_seq(new MDAT::ProteinSequence("seq1", "PTPAPGLFPSPIPASYLFP"));
		set.add_seq(new MDAT::ProteinSequence("seq2", "PTPAPGLFPSPIPASYLFPPRRRRRRRRRR"));
		determineRegionsToAlignWithFixedDomainOrder(reg1, reg2, mat, set[0].size(), set[1].size());
		/*for (int i=0;i<=set[0].size();++i)
		{
			for (int j=0; j<=set[1].size();++j)
				cout << " " << mat[i][j];
			cout << "\n";
		}
*/
		std::string matrix_path = getenv("HOME");
		matrix_path.append("/.mda/BLOSUM62.txt");
		MDAT::Matrix<int> *sim_mat = read_similarity_matrix(matrix_path.c_str());
		MDAT::MatrixStack<3,std::pair<float, char> > matrices(set[0].size()+1, set[1].size()+1);
		std::vector<size_t> ids1(1,0);
		std::vector<size_t> ids2(1,1);
		fillGotohMatrix(set, ids1, ids2, matrices, *sim_mat, mat);
		gotoh_align(set[0].size(), set[1].size(), matrices, -11, -1, mat);
		std::string edit_string1, edit_string2;
		gotoh_traceback(set[0].size(), set[1].size(), matrices, edit_string1, edit_string2);
		//cout << edit_string1 << "\n"  << edit_string2 << endl;
	/*	for (int i=0;i<=set[0].size();++i)
		{
			for (int j=0; j<=set[1].size();++j)
				cout << " " << matrices[0][i][j].first;
			cout << "\n";
		}*/
		set[0].insert_gaps(edit_string1);
		set[1].insert_gaps(edit_string2);
		TS_ASSERT_EQUALS(set[0].sequence(), "PT------PAPGLFPSPIPASYLFP-----------");
		TS_ASSERT_EQUALS(set[1].sequence(), "PTPAPGLFP------SPIPASYLFPPRRRRRRRRRR");
	}


};


#endif /* ALIGN_TEST_HPP_ */
